"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const Path = __importStar(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const Exception_1 = require("../../Exception");
const ToTypeNode = __importStar(require("./toTypeNode"));
const generatePath = (entryPoint, currentPoint, referencePath) => {
    const ext = Path.extname(currentPoint); // .yml
    const from = Path.relative(Path.dirname(entryPoint), currentPoint).replace(ext, ""); // components/schemas/A/B
    const base = Path.dirname(from);
    const result = Path.relative(base, referencePath); // remoteの場合? localの場合 referencePath.split("/")
    const pathArray = result.split("/");
    return {
        pathArray,
        base,
    };
};
const generateName = (store, base, pathArray) => {
    let names = [];
    pathArray.reduce((previous, lastPath, index) => {
        const current = Path.join(previous, lastPath);
        // ディレクトリが深い場合は相対パスが`..`を繰り返す可能性があり、
        // その場合はすでに登録されたnamesを削除する
        if (lastPath === ".." && names.length > 0) {
            names = names.slice(0, names.length - 1);
        }
        const isLast = index === pathArray.length - 1;
        if (isLast) {
            const statement = store.getStatement(current, "interface");
            const statement2 = store.getStatement(current, "typeAlias");
            const statement3 = store.getStatement(current, "namespace");
            if (statement) {
                names.push(statement.name);
            }
            else if (statement2) {
                names.push(statement2.name);
            }
            else if (statement3) {
                names.push(statement3.name);
            }
        }
        else {
            const statement = store.getStatement(current, "namespace");
            if (statement) {
                names.push(statement.value.name.text);
            }
        }
        return current;
    }, base);
    if (names.length === 0) {
        throw new Exception_1.DevelopmentError("Local Reference Error \n" + JSON.stringify({ pathArray, names, base }, null, 2));
    }
    return names.join(".");
};
const create = (entryPoint, store, factory) => {
    const getReferenceName = (currentPoint, referencePath) => {
        const { pathArray, base } = generatePath(entryPoint, currentPoint, referencePath);
        return generateName(store, base, pathArray);
    };
    const setReferenceHandler = reference => {
        if (store.hasStatement(reference.path, ["interface", "typeAlias"])) {
            return;
        }
        if (reference.type === "remote") {
            const typeNode = ToTypeNode.convert(entryPoint, reference.referencePoint, factory, reference.data, {
                setReferenceHandler,
                getReferenceName,
            });
            if (typescript_1.default.isTypeLiteralNode(typeNode)) {
                store.addStatement(reference.path, {
                    type: "interface",
                    name: reference.name,
                    value: factory.InterfaceDeclaration.create({
                        export: true,
                        name: reference.name,
                        members: typeNode.members,
                    }),
                });
            }
            else {
                const value = factory.TypeAliasDeclaration.create({
                    export: true,
                    name: reference.name,
                    type: ToTypeNode.convert(entryPoint, reference.referencePoint, factory, reference.data, {
                        setReferenceHandler,
                        getReferenceName,
                    }),
                });
                store.addStatement(reference.path, {
                    name: reference.name,
                    type: "typeAlias",
                    value,
                });
            }
        }
        else if (reference.type === "local") {
            if (!store.hasStatement(reference.path, ["namespace", "interface", "typeAlias"])) {
                throw new Exception_1.NotFoundReference(`The schema ${reference.name} is undefined in "${reference.path}".`);
            }
        }
    };
    return { setReferenceHandler: setReferenceHandler, getReferenceName };
};
exports.create = create;
//# sourceMappingURL=Context.js.map