"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateApiClientCode = void 0;
const Name = __importStar(require("./Name"));
const extractPickedParameter = (parameter) => {
    return {
        name: parameter.name,
        in: parameter.in,
        required: parameter.required,
        style: parameter.style,
        explode: parameter.explode,
    };
};
const extractSuccessStatusCode = (responses) => {
    const statusCodeList = [];
    Object.entries(responses || {}).forEach(([statusCodeLike, response]) => {
        // ContentTypeの定義が存在しない場合はstatusCodeを読み取らない
        if (Object.keys(response.content || {}).length === 0) {
            return;
        }
        if (typeof statusCodeLike === "string") {
            const statusCodeNumberValue = parseInt(statusCodeLike, 10);
            if (200 <= statusCodeNumberValue && statusCodeNumberValue < 300) {
                statusCodeList.push(statusCodeNumberValue.toString());
            }
        }
    });
    return statusCodeList;
};
const getRequestContentTypeList = (requestBody) => {
    return Object.keys(requestBody.content);
};
const getSuccessResponseContentTypeList = (responses) => {
    let contentTypeList = [];
    extractSuccessStatusCode(responses).forEach(statusCode => {
        const response = responses[statusCode];
        contentTypeList = contentTypeList.concat(Object.keys(response.content || {}));
    });
    return Array.from(new Set(contentTypeList));
};
const hasQueryParameters = (parameters) => {
    if (!parameters) {
        return false;
    }
    return parameters.filter(parameter => parameter.in === "query").length > 0;
};
const generateCodeGeneratorParamsList = (store) => {
    const operationState = store.getNoReferenceOperationState();
    const params = [];
    Object.entries(operationState).forEach(([operationId, item]) => {
        const responseSuccessNames = extractSuccessStatusCode(item.responses).map(statusCode => Name.responseName(operationId, statusCode));
        const requestContentTypeList = item.requestBody ? getRequestContentTypeList(item.requestBody) : [];
        const responseSuccessContentTypes = getSuccessResponseContentTypeList(item.responses);
        const hasOver2RequestContentTypes = requestContentTypeList.length > 1;
        const hasOver2SuccessNames = responseSuccessNames.length > 1;
        const formatParams = {
            operationId: operationId,
            rawRequestUri: item.requestUri,
            httpMethod: item.httpMethod,
            argumentParamsTypeDeclaration: Name.argumentParamsTypeDeclaration(operationId),
            // function
            functionName: operationId,
            comment: item.comment,
            deprecated: item.deprecated,
            //
            hasRequestBody: !!item.requestBody,
            hasParameter: item.parameters ? item.parameters.length > 0 : false,
            pickedParameters: item.parameters ? item.parameters.map(extractPickedParameter) : [],
            // Request Content Types
            requestContentTypes: requestContentTypeList,
            requestFirstContentType: requestContentTypeList.length === 1 ? requestContentTypeList[0] : undefined,
            has2OrMoreRequestContentTypes: hasOver2RequestContentTypes,
            // Response Success Name
            responseSuccessNames: responseSuccessNames,
            responseFirstSuccessName: responseSuccessNames.length === 1 ? responseSuccessNames[0] : undefined,
            has2OrMoreSuccessNames: hasOver2SuccessNames,
            // Response Success Content Type
            successResponseContentTypes: responseSuccessContentTypes,
            successResponseFirstContentType: responseSuccessContentTypes.length === 1 ? responseSuccessContentTypes[0] : undefined,
            has2OrMoreSuccessResponseContentTypes: responseSuccessContentTypes.length > 1,
            //
            hasAdditionalHeaders: hasOver2RequestContentTypes || hasOver2SuccessNames,
            hasQueryParameters: hasQueryParameters(item.parameters),
        };
        params.push(formatParams);
    });
    return params;
};
const generateApiClientCode = (store, context, makeApiClient) => {
    const codeGeneratorParamsList = generateCodeGeneratorParamsList(store);
    store.addAdditionalStatement(makeApiClient(context, codeGeneratorParamsList));
};
exports.generateApiClientCode = generateApiClientCode;
//# sourceMappingURL=Generator.js.map