"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateAliasInterface = exports.generateInterface = exports.generatePropertySignatures = exports.generatePropertySignature = exports.generateTypeAlias = exports.generateTypeNode = void 0;
const Guard = __importStar(require("../Guard"));
const Name = __importStar(require("../Name"));
const ToTypeNode = __importStar(require("../toTypeNode"));
const Reference = __importStar(require("./Reference"));
const generateTypeNode = (entryPoint, currentPoint, factory, parameter, context) => {
    return ToTypeNode.convert(entryPoint, currentPoint, factory, parameter.schema || { type: "null" }, context);
};
exports.generateTypeNode = generateTypeNode;
const generateTypeAlias = (entryPoint, currentPoint, factory, name, parameter, context) => {
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        comment: parameter.description,
        type: exports.generateTypeNode(entryPoint, currentPoint, factory, parameter, context),
    });
};
exports.generateTypeAlias = generateTypeAlias;
const generatePropertySignature = (entryPoint, currentPoint, factory, parameter, context) => {
    if (Guard.isReference(parameter)) {
        const reference = Reference.generate(entryPoint, currentPoint, parameter);
        if (reference.type === "local") {
            context.setReferenceHandler(reference);
            return factory.PropertySignature.create({
                name: reference.name,
                optional: false,
                type: factory.TypeReferenceNode.create({
                    name: context.getReferenceName(currentPoint, reference.path),
                }),
            });
        }
        const isPathProperty = reference.data.in === "path";
        return factory.PropertySignature.create({
            name: reference.data.name,
            optional: isPathProperty ? false : !reference.data.required,
            type: ToTypeNode.convert(entryPoint, reference.referencePoint, factory, reference.data.schema || { type: "null" }, context),
        });
    }
    const isPathProperty = parameter.in === "path";
    return factory.PropertySignature.create({
        name: Name.escapeText(parameter.name),
        optional: isPathProperty ? false : !parameter.required,
        type: ToTypeNode.convert(entryPoint, currentPoint, factory, parameter.schema || { type: "null" }, context),
    });
};
exports.generatePropertySignature = generatePropertySignature;
const generatePropertySignatures = (entryPoint, currentPoint, factory, parameters, context) => {
    return parameters.map(parameter => {
        return exports.generatePropertySignature(entryPoint, currentPoint, factory, parameter, context);
    });
};
exports.generatePropertySignatures = generatePropertySignatures;
const generateInterface = (entryPoint, currentPoint, factory, name, parameters, context) => {
    return factory.InterfaceDeclaration.create({
        export: true,
        name,
        comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#headerObject`,
        members: exports.generatePropertySignatures(entryPoint, currentPoint, factory, parameters, context),
    });
};
exports.generateInterface = generateInterface;
/**
 * Alias作成用
 */
const generateAliasInterface = (entryPoint, currentPoint, factory, name, parameters, context) => {
    return factory.InterfaceDeclaration.create({
        export: true,
        name,
        comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#headerObject`,
        members: exports.generatePropertySignatures(entryPoint, currentPoint, factory, parameters, context),
    });
};
exports.generateAliasInterface = generateAliasInterface;
//# sourceMappingURL=Parameter.js.map