"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNamespaceWithList = exports.generateNamespace = void 0;
const Exception_1 = require("../../../Exception");
const Guard = __importStar(require("../Guard"));
const Name = __importStar(require("../Name"));
const Paramter = __importStar(require("./Parameter"));
const Reference = __importStar(require("./Reference"));
const Schema = __importStar(require("./Schema"));
const generateNamespace = (entryPoint, currentPoint, store, factory, parameters, context) => {
    const basePath = "components/parameters";
    store.addComponent("parameters", {
        type: "namespace",
        name: Name.Components.Parameters,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.Parameters,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#parameterObject`,
        }),
        statements: {},
    });
    Object.entries(parameters).forEach(([name, parameter]) => {
        if (Guard.isReference(parameter)) {
            const reference = Reference.generate(entryPoint, currentPoint, parameter);
            if (reference.type === "local") {
                throw new Exception_1.UnSupportError("What is components.parameters local reference?");
            }
            else {
                if (!reference.data.schema) {
                    return;
                }
                Schema.addSchema(entryPoint, currentPoint, store, factory, reference.path, reference.name, reference.data.schema, context);
                store.addStatement(`${basePath}/${name}`, {
                    type: "typeAlias",
                    name: name,
                    value: factory.TypeAliasDeclaration.create({
                        export: true,
                        name: name,
                        type: factory.TypeReferenceNode.create({
                            name: context.getReferenceName(currentPoint, reference.path),
                        }),
                    }),
                });
            }
        }
        else {
            const path = `${basePath}/${name}`;
            store.addStatement(path, {
                type: "typeAlias",
                name: name,
                value: Paramter.generateTypeAlias(entryPoint, currentPoint, factory, name, parameter, context),
            });
        }
    });
};
exports.generateNamespace = generateNamespace;
const generateNamespaceWithList = (entryPoint, currentPoint, store, factory, parameters, context) => {
    store.addComponent("parameters", {
        type: "namespace",
        name: Name.Components.Parameters,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.Parameters,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#parameterObject`,
        }),
        statements: {},
    });
    parameters.forEach(parameter => {
        if (Guard.isReference(parameter)) {
            const reference = Reference.generate(entryPoint, currentPoint, parameter);
            if (reference.type === "local") {
                throw new Exception_1.UnSupportError("What is components.parameters local reference?");
            }
            const path = `components/parameters/${reference.name}`;
            return store.addStatement(path, {
                type: "typeAlias",
                name: reference.name,
                value: Paramter.generateTypeAlias(entryPoint, reference.referencePoint, factory, reference.name, reference.data, context),
            });
        }
        const path = `components/parameters/${parameter.name}`;
        return store.addStatement(path, {
            type: "typeAlias",
            name: parameter.name,
            value: Paramter.generateTypeAlias(entryPoint, currentPoint, factory, parameter.name, parameter, context),
        });
    });
};
exports.generateNamespaceWithList = generateNamespaceWithList;
//# sourceMappingURL=Parameters.js.map