"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveLocalReference = exports.resolveRemoteReference = exports.generate = exports.generateReferencePoint = exports.generateLocalReference = exports.getLocalReferencePattern = exports.localReferenceComponents = void 0;
const path = __importStar(require("path"));
const Exception_1 = require("../../../Exception");
const FileSystem_1 = require("../../../FileSystem");
const Logger = __importStar(require("../../../Logger"));
const Guard = __importStar(require("../Guard"));
const localReferencePatterns = [
    "#/components/schemas/",
    "#/components/responses/",
    "#/components/parameters/",
    // "#/components/examples/",
    "#/components/requestBodies/",
    "#/components/headers/",
    "#/components/securitySchemes/",
    // "#/components/links/",
    // "#/components/callbacks/",
    "#/components/pathItems/",
];
exports.localReferenceComponents = {
    "#/components/schemas/": "components/schemas",
    "#/components/responses/": "components/responses",
    "#/components/parameters/": "components/parameters",
    // "#/components/examples/": "components/examples",
    "#/components/requestBodies/": "components/requestBodies",
    "#/components/headers/": "components/headers",
    "#/components/securitySchemes/": "components/securitySchemes",
    // "#/components/links/": "components/links",
    // "#/components/callbacks/": "components/callbacks",
    "#/components/pathItems/": "components/pathItems",
};
const getLocalReferencePattern = (reference) => {
    let localReferencePattern;
    localReferencePatterns.forEach(referencePattern => {
        if (new RegExp("^" + referencePattern).test(reference.$ref)) {
            localReferencePattern = referencePattern;
        }
    });
    return localReferencePattern;
};
exports.getLocalReferencePattern = getLocalReferencePattern;
const generateLocalReference = (reference) => {
    const localReferencePattern = exports.getLocalReferencePattern(reference);
    if (!localReferencePattern) {
        return;
    }
    const name = reference.$ref.split(localReferencePattern)[1];
    const localPath = path.join(exports.localReferenceComponents[localReferencePattern], name);
    if (!localPath.startsWith("components")) {
        throw new Exception_1.DevelopmentError(`localPath is not start "components":\n${localPath}`);
    }
    return {
        type: "local",
        name,
        path: localPath,
    };
};
exports.generateLocalReference = generateLocalReference;
const generateReferencePoint = (currentPoint, reference) => {
    const basedir = path.dirname(currentPoint);
    const ref = reference.$ref;
    const referencePoint = path.join(basedir, ref);
    return referencePoint;
};
exports.generateReferencePoint = generateReferencePoint;
const generate = (entryPoint, currentPoint, reference) => {
    const localReference = exports.generateLocalReference(reference);
    if (localReference) {
        return localReference;
    }
    if (reference.$ref.startsWith("http")) {
        throw new Exception_1.FeatureDevelopmentError("Please Pull Request ! Welcome !");
    }
    const referencePoint = exports.generateReferencePoint(currentPoint, reference);
    if (!FileSystem_1.fileSystem.existSync(referencePoint)) {
        Logger.showFilePosition(entryPoint, currentPoint, referencePoint);
        Logger.error(JSON.stringify(reference, null, 2));
        throw new Exception_1.NotFoundFileError(`Not found reference point from current point. \n Path: ${referencePoint}`);
    }
    const relativePathFromEntryPoint = path.relative(path.dirname(entryPoint), referencePoint); // components/hoge/fuga.yml
    const ext = path.extname(relativePathFromEntryPoint); // .yml
    const pathArray = relativePathFromEntryPoint.replace(ext, "").split("/"); // ["components", "hoge", "fuga"]
    const targetPath = pathArray.join("/"); // components/hoge/fuga
    const schemaName = pathArray[pathArray.length - 1]; // fuga
    const componentName = pathArray[0] === "components" ? pathArray[1] : "";
    const data = FileSystem_1.fileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return exports.generate(entryPoint, referencePoint, data);
    }
    if (!targetPath.startsWith("components")) {
        throw new Exception_1.DevelopmentError(`targetPath is not start "components":\n${targetPath}`);
    }
    return {
        type: "remote",
        referencePoint,
        path: targetPath,
        name: schemaName,
        componentName: Guard.isComponentName(componentName) ? componentName : undefined,
        data,
    };
};
exports.generate = generate;
const resolveRemoteReference = (entryPoint, currentPoint, reference) => {
    if (reference.$ref.startsWith("#") || reference.$ref.startsWith("http")) {
        return { referencePoint: currentPoint, data: reference };
    }
    const referencePoint = exports.generateReferencePoint(currentPoint, reference);
    if (!FileSystem_1.fileSystem.existSync(referencePoint)) {
        Logger.showFilePosition(entryPoint, currentPoint, referencePoint);
        Logger.error(JSON.stringify(reference, null, 2));
        throw new Exception_1.NotFoundFileError(`Not found reference point from current point. \n Path: ${referencePoint}`);
    }
    const data = FileSystem_1.fileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return exports.resolveRemoteReference(entryPoint, referencePoint, data);
    }
    return {
        referencePoint,
        data,
    };
};
exports.resolveRemoteReference = resolveRemoteReference;
const resolveLocalReference = (entryPoint, currentPoint, reference) => {
    if (!reference.$ref.startsWith("#")) {
        return reference;
    }
    const referencePoint = exports.generateReferencePoint(currentPoint, reference);
    const data = FileSystem_1.fileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return exports.resolveRemoteReference(entryPoint, referencePoint, data);
    }
    return data;
};
exports.resolveLocalReference = resolveLocalReference;
//# sourceMappingURL=Reference.js.map