"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addSchema = exports.generateMultiTypeAlias = exports.generateTypeAlias = exports.generateArrayTypeAlias = exports.generateInterface = exports.generatePropertySignatures = void 0;
const Exception_1 = require("../../../Exception");
const Guard = __importStar(require("../Guard"));
const ToTypeNode = __importStar(require("../toTypeNode"));
const ExternalDocumentation = __importStar(require("./ExternalDocumentation"));
const generatePropertySignatures = (entryPoint, currentPoint, factory, schema, context) => {
    if (!schema.properties) {
        return [];
    }
    const required = schema.required || [];
    return Object.entries(schema.properties).map(([propertyName, property]) => {
        return factory.PropertySignature.create({
            name: propertyName,
            optional: !required.includes(propertyName),
            type: ToTypeNode.convert(entryPoint, currentPoint, factory, property, context, { parent: schema }),
            comment: typeof property !== "boolean" ? property.description : undefined,
        });
    });
};
exports.generatePropertySignatures = generatePropertySignatures;
const generateInterface = (entryPoint, currentPoint, factory, name, schema, context) => {
    if (schema.type !== "object") {
        throw new Exception_1.FeatureDevelopmentError("Please use generateTypeAlias");
    }
    let members = [];
    const propertySignatures = exports.generatePropertySignatures(entryPoint, currentPoint, factory, schema, context);
    if (Guard.isObjectSchemaWithAdditionalProperties(schema)) {
        const additionalProperties = ToTypeNode.convertAdditionalProperties(entryPoint, currentPoint, factory, schema, context);
        if (schema.additionalProperties === true) {
            members = members.concat(additionalProperties);
        }
        else {
            members = [...propertySignatures, additionalProperties];
        }
    }
    else {
        members = propertySignatures;
    }
    return factory.InterfaceDeclaration.create({
        export: true,
        name,
        members,
        comment: ExternalDocumentation.addComment(schema.description, schema.externalDocs),
    });
};
exports.generateInterface = generateInterface;
const generateArrayTypeAlias = (entryPoint, currentPoint, factory, name, schema, context) => {
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        comment: schema.description,
        type: ToTypeNode.convert(entryPoint, currentPoint, factory, schema, context),
    });
};
exports.generateArrayTypeAlias = generateArrayTypeAlias;
const generateTypeAlias = (entryPoint, currentPoint, factory, name, schema) => {
    let type;
    if (schema.enum) {
        if (Guard.isNumberArray(schema.enum) && (schema.type === "number" || schema.type === "integer")) {
            type = factory.TypeNode.create({
                type: schema.type,
                enum: schema.enum,
            });
        }
        else if (Guard.isStringArray(schema.enum) && schema.type === "string") {
            type = factory.TypeNode.create({
                type: schema.type,
                enum: schema.enum,
            });
        }
        else {
            type = factory.TypeNode.create({
                type: schema.type,
            });
        }
    }
    else {
        type = factory.TypeNode.create({
            type: schema.type,
        });
    }
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        type,
    });
};
exports.generateTypeAlias = generateTypeAlias;
const generateMultiTypeAlias = (entryPoint, currentPoint, factory, name, schemas, context, multiType) => {
    const type = ToTypeNode.generateMultiTypeNode(entryPoint, currentPoint, factory, schemas, context, ToTypeNode.convert, multiType);
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        type,
    });
};
exports.generateMultiTypeAlias = generateMultiTypeAlias;
const addSchema = (entryPoint, currentPoint, store, factory, targetPoint, declarationName, schema, context) => {
    if (!schema) {
        return;
    }
    if (Guard.isAllOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: exports.generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.allOf, context, "allOf"),
        });
    }
    else if (Guard.isOneOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: exports.generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.oneOf, context, "oneOf"),
        });
    }
    else if (Guard.isAnyOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: exports.generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.anyOf, context, "allOf"),
        });
    }
    else if (Guard.isArraySchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: exports.generateArrayTypeAlias(entryPoint, currentPoint, factory, declarationName, schema, context),
        });
    }
    else if (Guard.isObjectSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "interface",
            name: declarationName,
            value: exports.generateInterface(entryPoint, currentPoint, factory, declarationName, schema, context),
        });
    }
    else if (Guard.isPrimitiveSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: exports.generateTypeAlias(entryPoint, currentPoint, factory, declarationName, schema),
        });
    }
};
exports.addSchema = addSchema;
//# sourceMappingURL=Schema.js.map