"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNamespace = void 0;
const Exception_1 = require("../../../Exception");
const Guard = __importStar(require("../Guard"));
const Name = __importStar(require("../Name"));
const Reference = __importStar(require("./Reference"));
const Schema = __importStar(require("./Schema"));
const generateNamespace = (entryPoint, currentPoint, store, factory, schemas, context) => {
    const basePath = "components/schemas";
    store.addComponent("schemas", {
        type: "namespace",
        name: Name.Components.Schemas,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.Schemas,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#schemaObject`,
        }),
        statements: {},
    });
    Object.entries(schemas).forEach(([name, schema]) => {
        if (Guard.isReference(schema)) {
            const reference = Reference.generate(entryPoint, currentPoint, schema);
            if (reference.type === "local") {
                const referenceName = context.getReferenceName(currentPoint, reference.path);
                store.addStatement(`${basePath}/${name}`, {
                    type: "typeAlias",
                    name: name,
                    value: factory.TypeAliasDeclaration.create({
                        export: true,
                        name: name,
                        type: factory.TypeReferenceNode.create({
                            name: referenceName,
                        }),
                    }),
                });
                return;
            }
            Schema.addSchema(entryPoint, reference.referencePoint, store, factory, reference.path, reference.name, reference.data, context);
            if (store.hasStatement(`${basePath}/${name}`, ["interface", "typeAlias"])) {
                return;
            }
            return store.addStatement(`${basePath}/${name}`, {
                type: "typeAlias",
                name,
                value: factory.TypeAliasDeclaration.create({
                    export: true,
                    name: name,
                    type: factory.TypeReferenceNode.create({
                        name: context.getReferenceName(currentPoint, reference.path),
                    }),
                }),
            });
        }
        const path = `${basePath}/${name}`;
        if (Guard.isAllOfSchema(schema)) {
            return store.addStatement(path, {
                type: "typeAlias",
                name,
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.allOf, context, "allOf"),
            });
        }
        if (Guard.isOneOfSchema(schema)) {
            return store.addStatement(path, {
                type: "typeAlias",
                name,
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.oneOf, context, "oneOf"),
            });
        }
        if (Guard.isAnyOfSchema(schema)) {
            return store.addStatement(path, {
                type: "typeAlias",
                name,
                value: Schema.generateMultiTypeAlias(entryPoint, currentPoint, factory, name, schema.anyOf, context, "anyOf"),
            });
        }
        if (Guard.isArraySchema(schema)) {
            return store.addStatement(path, {
                type: "typeAlias",
                name,
                value: Schema.generateArrayTypeAlias(entryPoint, currentPoint, factory, name, schema, context),
            });
        }
        if (Guard.isObjectSchema(schema)) {
            return store.addStatement(path, {
                type: "interface",
                name,
                value: Schema.generateInterface(entryPoint, currentPoint, factory, name, schema, context),
            });
        }
        if (Guard.isObjectSchema(schema)) {
            return store.addStatement(path, {
                type: "interface",
                name,
                value: Schema.generateInterface(entryPoint, currentPoint, factory, name, schema, context),
            });
        }
        if (Guard.isPrimitiveSchema(schema)) {
            return store.addStatement(path, {
                type: "typeAlias",
                name,
                value: Schema.generateTypeAlias(entryPoint, currentPoint, factory, name, schema),
            });
        }
        throw new Exception_1.UnSupportError("schema.type = Array[] not supported. " + JSON.stringify(schema));
    });
};
exports.generateNamespace = generateNamespace;
//# sourceMappingURL=Schemas.js.map