"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const os_1 = require("os");
const Exception_1 = require("../../../Exception");
const httpMethodList = ["get", "put", "post", "delete", "options", "head", "patch", "trace"];
const create = (rootSchema) => {
    if (!rootSchema.paths) {
        throw new Exception_1.DevelopmentError("pathsの存在しないところで利用しないでください");
    }
    const state = {};
    Object.entries(rootSchema.paths).forEach(([requestUri, pathItem]) => {
        httpMethodList.forEach(httpMethod => {
            const operation = pathItem[httpMethod];
            if (!operation) {
                return;
            }
            if (!operation.operationId) {
                return;
            }
            state[operation.operationId] = {
                httpMethod,
                requestUri,
                comment: [operation.summary, operation.description].filter(Boolean).join(os_1.EOL),
                deprecated: !!operation.deprecated,
                requestBody: operation.requestBody,
                parameters: operation.parameters,
                responses: operation.responses,
            };
        });
    });
    return state;
};
exports.create = create;
//# sourceMappingURL=Operation.js.map