"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const fs = __importStar(require("fs"));
const path_1 = require("path");
const dot_prop_1 = __importDefault(require("dot-prop"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const Exception_1 = require("../../../Exception");
const Def = __importStar(require("./Definition"));
const Masking = __importStar(require("./masking"));
const Operation = __importStar(require("./Operation"));
const PropAccess = __importStar(require("./PropAccess"));
const State = __importStar(require("./State"));
const create = (factory, rootDocument) => {
    const state = State.createDefaultState(rootDocument);
    const getPathItem = (localPath) => {
        if (!localPath.startsWith("components/pathItem")) {
            throw new Error("Only use start with 'component/pathItems': " + localPath);
        }
        const result = dot_prop_1.default.get(state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    };
    const createNamespace = (name) => {
        const value = factory.Namespace.create({
            export: true,
            name,
            statements: [],
        });
        return {
            type: "namespace",
            name,
            value,
            statements: {},
        };
    };
    const addComponent = (componentName, statement) => {
        const key = Def.generateKey("namespace", componentName);
        state.components[key] = statement;
    };
    const getStatement = (path, type) => {
        const targetPath = path_1.relative("components", path);
        const result = PropAccess.get(state.components, type, targetPath);
        return result;
    };
    const hasStatement = (path, types) => {
        const targetPath = path_1.relative("components", path);
        return types.some(type => !!PropAccess.get(state.components, type, targetPath));
    };
    const addStatement = (path, statement) => {
        if (!path.startsWith("components")) {
            throw new Exception_1.UnSupportError(`componentsから始まっていません。path=${path}`);
        }
        const targetPath = path_1.relative("components", path);
        state.components = PropAccess.set(state.components, targetPath, statement, createNamespace);
    };
    const interfaceToStatement = (node) => {
        return node.value;
    };
    const typeAliasToStatement = (node) => {
        return node.value;
    };
    const namespaceToTsStatement = (node) => {
        const statements = Object.values(node.statements).reduce((previous, childStatement) => {
            if (!childStatement) {
                return previous;
            }
            if (childStatement.type === "namespace") {
                return previous.concat(namespaceToTsStatement(childStatement));
            }
            if (childStatement.type === "typeAlias") {
                return previous.concat(typeAliasToStatement(childStatement));
            }
            return previous.concat(interfaceToStatement(childStatement));
        }, []);
        return factory.Namespace.addStatements({
            node: node.value,
            statements,
        });
    };
    const dump = (filename) => {
        fs.writeFileSync(filename, js_yaml_1.default.dump(Masking.maskValue(state)), { encoding: "utf-8" });
    };
    const dumpOperationState = (filename) => {
        fs.writeFileSync(filename, JSON.stringify(state.operations, null, 2), { encoding: "utf-8" });
    };
    const addAdditionalStatement = (statements) => {
        state.additionalStatements = state.additionalStatements.concat(statements);
    };
    const getRootStatements = () => {
        const statements = Def.componentNames.reduce((statements, componentName) => {
            const component = state.components[Def.generateKey("namespace", componentName)];
            if (!component) {
                return statements;
            }
            if (component.type === "interface") {
                return statements.concat(interfaceToStatement(component));
            }
            if (component.type === "namespace") {
                return statements.concat(namespaceToTsStatement(component));
            }
            if (component.type === "typeAlias") {
                return statements.concat(typeAliasToStatement(component));
            }
            return statements;
        }, []);
        return statements.concat(state.additionalStatements);
    };
    const updateOperationState = (httpMethod, requestUri, operationId, newOperationState) => {
        let operationState = state.operations[operationId];
        if (operationState) {
            operationState = { ...operationState, ...newOperationState };
        }
        else {
            operationState = State.createDefaultOperationState(httpMethod, requestUri, newOperationState);
        }
        state.operations[operationId] = operationState;
    };
    const getOperationState = (operationId) => {
        const operationState = state.operations[operationId];
        if (operationState) {
            return operationState;
        }
        dumpOperationState("debug/error-getOperationState.json");
        throw new Error(`Not found ${operationId}`);
    };
    const getNoReferenceOperationState = () => {
        return Operation.create(state.document);
    };
    return {
        hasStatement,
        addStatement,
        getStatement,
        getRootStatements,
        addComponent,
        dump,
        getNoReferenceOperationState,
        updateOperationState,
        getOperationState,
        addAdditionalStatement,
        dumpOperationState,
        getPathItem,
    };
};
exports.create = create;
//# sourceMappingURL=Store.js.map