"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertAdditionalProperties = exports.convert = exports.generateMultiTypeNode = void 0;
const Exception_1 = require("../../Exception");
const Exception_2 = require("../../Exception");
const Logger = __importStar(require("../../Logger"));
const Reference = __importStar(require("./components/Reference"));
const Guard = __importStar(require("./Guard"));
const generateMultiTypeNode = (entryPoint, currentPoint, factory, schemas, setReference, convert, multiType) => {
    const typeNodes = schemas.map(schema => convert(entryPoint, currentPoint, factory, schema, setReference));
    if (multiType === "oneOf") {
        return factory.UnionTypeNode.create({
            typeNodes,
        });
    }
    if (multiType === "allOf") {
        return factory.IntersectionTypeNode.create({
            typeNodes,
        });
    }
    // TODO Feature Development: Calculate intersection types
    return factory.TypeNode.create({ type: "never" });
};
exports.generateMultiTypeNode = generateMultiTypeNode;
const nullable = (factory, typeNode, nullable) => {
    if (nullable) {
        return factory.UnionTypeNode.create({
            typeNodes: [
                typeNode,
                factory.TypeNode.create({
                    type: "null",
                }),
            ],
        });
    }
    return typeNode;
};
const convert = (entryPoint, currentPoint, factory, schema, context, option) => {
    if (typeof schema === "boolean") {
        // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
        return factory.TypeNode.create({
            type: "object",
            value: [],
        });
    }
    if (Guard.isReference(schema)) {
        const reference = Reference.generate(entryPoint, currentPoint, schema);
        if (reference.type === "local") {
            // Type Aliasを作成 (or すでにある場合は作成しない)
            context.setReferenceHandler(reference);
            return factory.TypeReferenceNode.create({ name: context.getReferenceName(currentPoint, reference.path) });
        }
        // サポートしているディレクトリに対して存在する場合
        if (reference.componentName) {
            // Type AliasもしくはInterfaceを作成
            context.setReferenceHandler(reference);
            // Aliasを貼る
            return factory.TypeReferenceNode.create({ name: context.getReferenceName(currentPoint, reference.path) });
        }
        // サポートしていないディレクトリに存在する場合、直接Interface、もしくはTypeAliasを作成
        return exports.convert(entryPoint, reference.referencePoint, factory, reference.data, context, { parent: schema });
    }
    if (Guard.isOneOfSchema(schema)) {
        return exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.oneOf, context, exports.convert, "oneOf");
    }
    if (Guard.isAllOfSchema(schema)) {
        return exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.allOf, context, exports.convert, "allOf");
    }
    if (Guard.isAnyOfSchema(schema)) {
        return exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.anyOf, context, exports.convert, "anyOf");
    }
    // schema.type
    if (!schema.type) {
        if (option && option.parent) {
            Logger.info("Parent Schema:");
            Logger.info(JSON.stringify(option.parent));
        }
        Logger.showFilePosition(entryPoint, currentPoint);
        throw new Exception_1.UnsetTypeError("Please set 'type' or '$ref' property \n" + JSON.stringify(schema));
    }
    switch (schema.type) {
        case "boolean": {
            const typeNode = factory.TypeNode.create({
                type: "boolean",
            });
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "null": {
            return factory.TypeNode.create({
                type: schema.type,
            });
        }
        case "integer":
        case "number": {
            const items = schema.enum;
            let typeNode;
            if (items && Guard.isNumberArray(items)) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    enum: items,
                });
            }
            else {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "string": {
            const items = schema.enum;
            let typeNode;
            if (items && Guard.isStringArray(items)) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    enum: items,
                });
            }
            else {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "array": {
            if (Array.isArray(schema.items) || typeof schema.items === "boolean") {
                throw new Exception_2.UnSupportError(`schema.items = ${JSON.stringify(schema.items)}`);
            }
            const typeNode = factory.TypeNode.create({
                type: schema.type,
                value: schema.items
                    ? exports.convert(entryPoint, currentPoint, factory, schema.items, context, { parent: schema })
                    : factory.TypeNode.create({
                        type: "undefined",
                    }),
            });
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "object": {
            if (!schema.properties) {
                return factory.TypeNode.create({
                    type: "object",
                    value: [],
                });
            }
            let typeNode;
            const required = schema.required || [];
            // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
            if (schema.additionalProperties === true) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    value: [],
                });
            }
            else {
                const value = Object.entries(schema.properties).map(([name, jsonSchema]) => {
                    return factory.PropertySignature.create({
                        name,
                        type: exports.convert(entryPoint, currentPoint, factory, jsonSchema, context, { parent: schema.properties }),
                        optional: !required.includes(name),
                        comment: typeof jsonSchema !== "boolean" ? jsonSchema.description : undefined,
                    });
                });
                if (schema.additionalProperties) {
                    const additionalProperties = factory.IndexSignatureDeclaration.create({
                        name: "key",
                        type: exports.convert(entryPoint, currentPoint, factory, schema.additionalProperties, context, { parent: schema.properties }),
                    });
                    return factory.TypeNode.create({
                        type: schema.type,
                        value: [...value, additionalProperties],
                    });
                }
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    value,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        default:
            throw new Exception_2.UnknownError("what is this? \n" + JSON.stringify(schema, null, 2));
    }
};
exports.convert = convert;
const convertAdditionalProperties = (entryPoint, currentPoint, factory, schema, setReference) => {
    // // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
    if (schema.additionalProperties === true) {
        factory.TypeNode.create({
            type: schema.type,
            value: [],
        });
    }
    const additionalProperties = factory.IndexSignatureDeclaration.create({
        name: "key",
        type: exports.convert(entryPoint, currentPoint, factory, schema.additionalProperties, setReference, { parent: schema.properties }),
    });
    return additionalProperties;
};
exports.convertAdditionalProperties = convertAdditionalProperties;
//# sourceMappingURL=toTypeNode.js.map