"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateObjectLiteralExpression = exports.generateVariableIdentifier = exports.generateTemplateExpression = void 0;
const v3_1 = require("../Converter/v3");
const Exception_1 = require("../Exception");
const getTemplateSpan = (factory, currentIndex, nextIndex, lastIndex, currentItem, nextItem) => {
    // == last
    if (!nextItem) {
        return [
            factory.TemplateSpan.create({
                expression: currentItem.value,
                literal: factory.TemplateTail.create({
                    text: "",
                }),
            }),
        ];
    }
    if (nextItem.type === "property") {
        if (lastIndex === nextIndex) {
            return [
                factory.TemplateSpan.create({
                    expression: currentItem.value,
                    literal: factory.TemplateTail.create({
                        text: "",
                    }),
                }),
                factory.TemplateSpan.create({
                    expression: nextItem.value,
                    literal: factory.TemplateTail.create({
                        text: "",
                    }),
                }),
            ];
        }
        else {
            return [
                factory.TemplateSpan.create({
                    expression: currentItem.value,
                    literal: factory.TemplateTail.create({
                        text: "",
                    }),
                }),
                factory.TemplateSpan.create({
                    expression: nextItem.value,
                    literal: factory.TemplateMiddle.create({
                        text: "",
                    }),
                }),
            ];
        }
    }
    else {
        if (lastIndex === nextIndex) {
            return [
                factory.TemplateSpan.create({
                    expression: currentItem.value,
                    literal: factory.TemplateTail.create({
                        text: nextItem.value,
                    }),
                }),
            ];
        }
        else {
            return [
                factory.TemplateSpan.create({
                    expression: currentItem.value,
                    literal: factory.TemplateMiddle.create({
                        text: nextItem.value,
                    }),
                }),
            ];
        }
    }
};
/**
 *
 * ``
 * `a`
 * `${b}`
 * `a${b}`
 * `${a}${b}`
 * `${a}b${c}`
 * ``
 */
const generateTemplateExpression = (factory, list) => {
    if (list.length === 0) {
        return factory.NoSubstitutionTemplateLiteral.create({
            text: "",
        });
    }
    const templateHead = factory.TemplateHead.create({
        text: list[0].type === "property" ? "" : list[0].value,
    });
    const spanList = list.splice(1, list.length);
    if (spanList.length === 0 && list[0].type === "string") {
        return factory.NoSubstitutionTemplateLiteral.create({
            text: list[0].value,
        });
    }
    const lastIndex = spanList.length - 1;
    const restValue = lastIndex % 2;
    let templateSpans = [];
    for (let i = 0; i <= (lastIndex - restValue) / 2; i++) {
        if (spanList.length === 0) {
            continue;
        }
        const currentIndex = 2 * i;
        const nextIndex = 2 * i + 1;
        const currentItem = spanList[currentIndex];
        const nextItem = spanList[nextIndex];
        if (currentItem.type === "string") {
            throw new Exception_1.DevelopmentError("Logic Error");
        }
        templateSpans = templateSpans.concat(getTemplateSpan(factory, currentIndex, nextIndex, lastIndex, currentItem, nextItem));
    }
    return factory.TemplateExpression.create({
        head: templateHead,
        templateSpans: templateSpans,
    });
};
exports.generateTemplateExpression = generateTemplateExpression;
const generateVariableIdentifier = (factory, name) => {
    if (name.startsWith("/")) {
        throw new Error("can't start '/'. name=" + name);
    }
    const list = name.split(".");
    if (list.length === 1) {
        return factory.Identifier.create({
            name: name,
        });
    }
    const [n1, n2, ...rest] = list;
    const first = factory.PropertyAccessExpression.create({
        expression: n1,
        name: n2,
    });
    return rest.reduce((previous, current) => {
        if (v3_1.Name.isAvailableVariableName(current)) {
            return factory.PropertyAccessExpression.create({
                expression: previous,
                name: current,
            });
        }
        return factory.ElementAccessExpression.create({
            expression: previous,
            index: current,
        });
    }, first);
};
exports.generateVariableIdentifier = generateVariableIdentifier;
const generateObjectLiteralExpression = (factory, obj, extraProperties = []) => {
    const properties = Object.entries(obj).map(([key, item]) => {
        const initializer = item.type === "variable" ? exports.generateVariableIdentifier(factory, item.value) : factory.StringLiteral.create({ text: item.value });
        return factory.PropertyAssignment.create({
            name: v3_1.Name.escapeText(key),
            initializer,
        });
    });
    return factory.ObjectLiteralExpression.create({
        properties: extraProperties.concat(properties),
        multiLine: true,
    });
};
exports.generateObjectLiteralExpression = generateObjectLiteralExpression;
//# sourceMappingURL=utils.js.map