"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = exports.OpenApi = void 0;
const dot_prop_1 = __importDefault(require("dot-prop"));
const Reference = __importStar(require("../Converter/v3/components/Reference"));
const Guard = __importStar(require("../Converter/v3/Guard"));
const types_1 = require("../Converter/v3/types");
Object.defineProperty(exports, "OpenApi", { enumerable: true, get: function () { return types_1.OpenApi; } });
const Exception_1 = require("../Exception");
const isObject = (value) => {
    return !!value && value !== null && !Array.isArray(value) && typeof value === "object";
};
const isArray = (value) => {
    return !!value && Array.isArray(value);
};
const isRemoteReference = (obj) => {
    return isObject(obj) && typeof obj.$ref === "string" && !obj.$ref.startsWith("#") && !obj.$ref.startsWith("http");
};
const isLocalReference = (obj) => {
    return !isRemoteReference(obj);
};
const resolveRemoteReference = (entryPoint, currentPoint, obj, parentKey) => {
    // console.log(parentKey);
    if (Guard.isReference(obj)) {
        if (isLocalReference(obj)) {
            return obj;
        }
        if (isRemoteReference(obj)) {
            const { referencePoint, data } = Reference.resolveRemoteReference(entryPoint, currentPoint, obj);
            if (!data) {
                throw new Error(`not defined schema: ${obj.$ref}`);
            }
            if (isArray(data)) {
                return data.map((item, idx) => resolveRemoteReference(entryPoint, referencePoint, item, [parentKey, idx].join(".")));
            }
            if (isObject(data)) {
                Object.entries(data).forEach(([key, value]) => {
                    data[key] = resolveRemoteReference(entryPoint, referencePoint, value, [parentKey, key].join("."));
                });
                return data;
            }
        }
        return obj;
    }
    // ----- objがreferenceではない場合 ------
    if (isArray(obj)) {
        return obj.map((item, idx) => resolveRemoteReference(entryPoint, currentPoint, item, [parentKey, idx].join(".")));
    }
    if (isObject(obj)) {
        Object.entries(obj).forEach(([key, value]) => {
            obj[key] = resolveRemoteReference(entryPoint, currentPoint, value, [parentKey, key].join("."));
        });
    }
    return obj;
};
const resolveLocalReference = (entryPoint, currentPoint, obj, rootSchema, parentKey) => {
    // console.log(parentKey);
    if (Guard.isReference(obj)) {
        if (isRemoteReference(obj)) {
            throw new Exception_1.DevelopmentError("まずはremote referenceを解決してください\n" + JSON.stringify(obj, null, 2));
        }
        if (isLocalReference(obj)) {
            const ref = Reference.generateLocalReference(obj);
            if (!ref) {
                throw new Exception_1.DevelopmentError(`なにかが間違っている\n${JSON.stringify(ref, null, 2)}`);
            }
            return dot_prop_1.default.get(rootSchema, ref.path.replace(/\//g, "."));
        }
        return obj;
    }
    // ----- objがreferenceではない場合 ------
    if (isArray(obj)) {
        return obj.map((item, idx) => resolveLocalReference(entryPoint, currentPoint, item, rootSchema, [parentKey, idx].join(".")));
    }
    if (isObject(obj)) {
        Object.entries(obj).forEach(([key, value]) => {
            obj[key] = resolveLocalReference(entryPoint, currentPoint, value, rootSchema, [parentKey, key].join("."));
        });
    }
    return obj;
};
const resolve = (entryPoint, currentPoint, rootSchema) => {
    const obj1 = resolveRemoteReference(entryPoint, currentPoint, rootSchema, "remote");
    const obj2 = resolveLocalReference(entryPoint, currentPoint, obj1, obj1, "local");
    return obj2;
};
exports.resolve = resolve;
//# sourceMappingURL=index.js.map