import * as Guard from "../Guard";
import * as Name from "../Name";
import * as ToTypeNode from "../toTypeNode";
import * as Reference from "./Reference";
export const generateTypeNode = (entryPoint, currentPoint, factory, name, header, context) => {
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        type: ToTypeNode.convert(entryPoint, currentPoint, factory, header.schema || { type: "null" }, context),
    });
};
export const generatePropertySignature = (entryPoint, currentPoint, factory, name, header, context) => {
    if (Guard.isReference(header)) {
        const reference = Reference.generate(entryPoint, currentPoint, header);
        if (reference.type === "local") {
            context.setReferenceHandler(reference);
            return factory.PropertySignature.create({
                name,
                optional: false,
                type: factory.TypeReferenceNode.create({
                    name: context.getReferenceName(currentPoint, reference.path),
                }),
            });
        }
        return factory.PropertySignature.create({
            name,
            optional: false,
            type: factory.TypeReferenceNode.create({
                name: context.getReferenceName(currentPoint, reference.path),
            }),
        });
    }
    return factory.PropertySignature.create({
        name: Name.escapeText(name),
        optional: false,
        type: ToTypeNode.convert(entryPoint, currentPoint, factory, header.schema || { type: "null" }, context),
    });
};
export const generatePropertySignatures = (entryPoint, currentPoint, factory, headers, context) => {
    return Object.entries(headers).map(([headerName, header]) => {
        return generatePropertySignature(entryPoint, currentPoint, factory, headerName, header, context);
    });
};
export const generateInterface = (entryPoint, currentPoint, factory, name, headers, context) => {
    return factory.InterfaceDeclaration.create({
        export: true,
        name,
        comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#headerObject`,
        members: generatePropertySignatures(entryPoint, currentPoint, factory, headers, context),
    });
};
//# sourceMappingURL=Header.js.map