import { UndefinedComponent } from "../../../Exception";
import * as Guard from "../Guard";
import * as Name from "../Name";
import * as Header from "./Header";
import * as Reference from "./Reference";
import * as Schema from "./Schema";
export const generateNamespace = (entryPoint, currentPoint, store, factory, headers, context) => {
    store.addComponent("headers", {
        type: "namespace",
        name: Name.Components.Headers,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.Headers,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#componentsObject`,
        }),
        statements: {},
    });
    Object.entries(headers).forEach(([name, header]) => {
        if (Guard.isReference(header)) {
            const reference = Reference.generate(entryPoint, currentPoint, header);
            if (reference.type === "local") {
                if (!store.hasStatement(reference.path, ["interface"])) {
                    throw new UndefinedComponent(`Reference "${header.$ref}" did not found in ${reference.path} by ${reference.name}`);
                }
            }
            else if (reference.type === "remote") {
                Schema.addSchema(entryPoint, currentPoint, store, factory, reference.path, reference.name, reference.data.schema, context);
            }
        }
        else {
            store.addStatement(`components/headers/${name}`, {
                type: "typeAlias",
                name: name,
                value: Header.generateTypeNode(entryPoint, currentPoint, factory, name, header, context),
            });
        }
    });
};
//# sourceMappingURL=Headers.js.map