import * as path from "path";
import * as Guard from "../Guard";
import * as Name from "../Name";
import * as Reference from "./Reference";
import * as RequestBody from "./RequestBody";
export const generateNamespace = (entryPoint, currentPoint, store, factory, requestBodies, context) => {
    const basePath = "components/requestBodies";
    store.addComponent("requestBodies", {
        type: "namespace",
        name: Name.Components.RequestBodies,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.RequestBodies,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#componentsObject`,
        }),
        statements: {},
    });
    Object.entries(requestBodies).forEach(([name, requestBody]) => {
        if (Guard.isReference(requestBody)) {
            const reference = Reference.generate(entryPoint, currentPoint, requestBody);
            if (reference.type === "local") {
                throw new Error("not support");
            }
            else if (reference.type === "remote") {
                RequestBody.generateNamespace(entryPoint, reference.referencePoint, store, factory, path.dirname(reference.path), reference.name, reference.data, context);
            }
        }
        else {
            RequestBody.generateNamespace(entryPoint, currentPoint, store, factory, basePath, name, requestBody, context);
        }
    });
};
//# sourceMappingURL=RequestBodies.js.map