import * as path from "path";
import * as Name from "../Name";
import * as Header from "./Header";
import * as MediaType from "./MediaType";
export const generateNamespace = (entryPoint, currentPoint, store, factory, parentPath, name, response, context) => {
    const basePath = `${parentPath}/${name}`;
    store.addStatement(basePath, {
        type: "namespace",
        name,
        value: factory.Namespace.create({
            export: true,
            name: name,
            comment: response.description,
            statements: [],
        }),
        statements: {},
    });
    if (response.headers) {
        store.addStatement(`${basePath}/Header`, {
            type: "interface",
            name: Name.ComponentChild.Header,
            value: Header.generateInterface(entryPoint, currentPoint, factory, Name.ComponentChild.Header, response.headers, context),
        });
    }
    if (response.content) {
        store.addStatement(`${basePath}/Content`, {
            type: "interface",
            name: Name.ComponentChild.Content,
            value: MediaType.generateInterface(entryPoint, currentPoint, factory, Name.ComponentChild.Content, response.content, context),
        });
    }
};
export const generateReferenceNamespace = (entryPoint, currentPoint, store, factory, parentPath, nameWithStatusCode, responseReference, context) => {
    const basePath = `${parentPath}/${nameWithStatusCode}`;
    const referenceNamespaceName = context.getReferenceName(currentPoint, responseReference.path);
    store.addStatement(basePath, {
        type: "namespace",
        name: nameWithStatusCode,
        value: factory.Namespace.create({
            export: true,
            name: nameWithStatusCode,
            statements: [],
        }),
        statements: {},
    });
    // TODO Type Guard
    const headerNamespace = store.getStatement(path.join(responseReference.path, "Header"), "namespace");
    if (headerNamespace) {
        store.addStatement(`${basePath}/Header`, {
            type: "namespace",
            name: Name.ComponentChild.Header,
            value: factory.Namespace.create({
                export: true,
                name: Name.ComponentChild.Header,
                statements: [],
            }),
            statements: {},
        });
        Object.values(headerNamespace.statements).forEach(statement => {
            if (!statement) {
                return;
            }
            if (statement.type === "interface" || statement.type === "typeAlias") {
                const aliasName = [referenceNamespaceName, Name.ComponentChild.Header, statement.name].join(".");
                store.addStatement(`${basePath}/Header/${statement.value.name.text}`, {
                    type: "typeAlias",
                    name: aliasName,
                    value: factory.TypeAliasDeclaration.create({
                        export: true,
                        name: statement.value.name.text,
                        type: factory.TypeReferenceNode.create({
                            name: aliasName,
                        }),
                    }),
                });
            }
        });
    }
    store.addStatement(`${basePath}/Content`, {
        type: "typeAlias",
        name: Name.ComponentChild.Content,
        value: factory.TypeAliasDeclaration.create({
            export: true,
            name: Name.ComponentChild.Content,
            type: factory.TypeReferenceNode.create({
                name: referenceNamespaceName + "." + Name.ComponentChild.Content,
            }),
        }),
    });
};
//# sourceMappingURL=Response.js.map