import * as Def from "./Definition";
const maskingNamespace = (namespaceStatement) => {
    const statements = Object.entries(namespaceStatement.statements).reduce((newStatement, [key, statement]) => {
        if (statement) {
            newStatement[key] = maskingStatement(statement);
        }
        return newStatement;
    }, {});
    return {
        type: "namespace",
        name: namespaceStatement.name,
        value: namespaceStatement.value.name.text,
        statements,
    };
};
const maskingInterface = (statement) => {
    return {
        type: "interface",
        name: statement.name,
        value: statement.value.name.text,
    };
};
const maskingTypeAlias = (statement) => {
    return {
        type: "typeAlias",
        name: statement.name,
        value: statement.value.name.text,
    };
};
const maskingStatement = (statement) => {
    if (statement.type === "interface") {
        return maskingInterface(statement);
    }
    if (statement.type === "typeAlias") {
        return maskingTypeAlias(statement);
    }
    return maskingNamespace(statement);
};
export const maskValue = (state) => {
    const kinds = ["namespace", "typeAlias", "interface"];
    return Def.componentNames.reduce((newState, componentName) => {
        kinds.forEach(kind => {
            const key = Def.generateKey(kind, componentName);
            const component = state.components[key];
            if (!component) {
                return;
            }
            newState.components[key] = maskingStatement(component);
        });
        return newState;
    }, state);
};
//# sourceMappingURL=masking.js.map