import DotProp from "dot-prop";
import * as Reference from "../Converter/v3/components/Reference";
import * as Guard from "../Converter/v3/Guard";
import { OpenApi } from "../Converter/v3/types";
import { DevelopmentError } from "../Exception";
export { OpenApi };
const isObject = (value) => {
    return !!value && value !== null && !Array.isArray(value) && typeof value === "object";
};
const isArray = (value) => {
    return !!value && Array.isArray(value);
};
const isRemoteReference = (obj) => {
    return isObject(obj) && typeof obj.$ref === "string" && !obj.$ref.startsWith("#") && !obj.$ref.startsWith("http");
};
const isLocalReference = (obj) => {
    return !isRemoteReference(obj);
};
const resolveRemoteReference = (entryPoint, currentPoint, obj, parentKey) => {
    // console.log(parentKey);
    if (Guard.isReference(obj)) {
        if (isLocalReference(obj)) {
            return obj;
        }
        if (isRemoteReference(obj)) {
            const { referencePoint, data } = Reference.resolveRemoteReference(entryPoint, currentPoint, obj);
            if (!data) {
                throw new Error(`not defined schema: ${obj.$ref}`);
            }
            if (isArray(data)) {
                return data.map((item, idx) => resolveRemoteReference(entryPoint, referencePoint, item, [parentKey, idx].join(".")));
            }
            if (isObject(data)) {
                Object.entries(data).forEach(([key, value]) => {
                    data[key] = resolveRemoteReference(entryPoint, referencePoint, value, [parentKey, key].join("."));
                });
                return data;
            }
        }
        return obj;
    }
    // ----- objがreferenceではない場合 ------
    if (isArray(obj)) {
        return obj.map((item, idx) => resolveRemoteReference(entryPoint, currentPoint, item, [parentKey, idx].join(".")));
    }
    if (isObject(obj)) {
        Object.entries(obj).forEach(([key, value]) => {
            obj[key] = resolveRemoteReference(entryPoint, currentPoint, value, [parentKey, key].join("."));
        });
    }
    return obj;
};
const resolveLocalReference = (entryPoint, currentPoint, obj, rootSchema, parentKey) => {
    // console.log(parentKey);
    if (Guard.isReference(obj)) {
        if (isRemoteReference(obj)) {
            throw new DevelopmentError("まずはremote referenceを解決してください\n" + JSON.stringify(obj, null, 2));
        }
        if (isLocalReference(obj)) {
            const ref = Reference.generateLocalReference(obj);
            if (!ref) {
                throw new DevelopmentError(`なにかが間違っている\n${JSON.stringify(ref, null, 2)}`);
            }
            return DotProp.get(rootSchema, ref.path.replace(/\//g, "."));
        }
        return obj;
    }
    // ----- objがreferenceではない場合 ------
    if (isArray(obj)) {
        return obj.map((item, idx) => resolveLocalReference(entryPoint, currentPoint, item, rootSchema, [parentKey, idx].join(".")));
    }
    if (isObject(obj)) {
        Object.entries(obj).forEach(([key, value]) => {
            obj[key] = resolveLocalReference(entryPoint, currentPoint, value, rootSchema, [parentKey, key].join("."));
        });
    }
    return obj;
};
export const resolve = (entryPoint, currentPoint, rootSchema) => {
    const obj1 = resolveRemoteReference(entryPoint, currentPoint, rootSchema, "remote");
    const obj2 = resolveLocalReference(entryPoint, currentPoint, obj1, obj1, "local");
    return obj2;
};
//# sourceMappingURL=index.js.map