export declare type InterfaceKey = string;
export declare type NamespaceKey = string;
export declare type Keys = InterfaceKey | NamespaceKey;
export interface NamespaceStatement<T, U, V> {
    type: "namespace";
    name: string;
    value: T;
    statements: StatementMap<T, U, V>;
}
export interface TypeAliasStatement<T> {
    type: "typeAlias";
    name: string;
    value: T;
}
export interface InterfaceStatement<T> {
    type: "interface";
    name: string;
    value: T;
}
export declare type Statement<T, U, V> = NamespaceStatement<T, U, V> | InterfaceStatement<U> | TypeAliasStatement<V>;
export interface StatementMap<T, U, V> {
    [key: string]: Statement<T, U, V> | undefined;
}
export declare type ComponentName = "schemas" | "headers" | "responses" | "parameters" | "requestBodies" | "securitySchemes" | "pathItems";
export declare type GenerateKey<A, B, C, T extends Statement<A, B, C>["type"]> = T extends "namespace" ? NamespaceKey : T extends "interface" ? InterfaceKey : never;
export declare type GetStatement<A, B, C, T extends Statement<A, B, C>["type"]> = T extends "namespace" ? NamespaceStatement<A, B, C> : T extends "interface" ? InterfaceStatement<B> : T extends "typeAlias" ? TypeAliasStatement<C> : never;
export declare const generateKey: <A, B, C, T extends "namespace" | "typeAlias" | "interface">(type: T, key: string) => GenerateKey<A, B, C, T>;
export declare const componentNames: ComponentName[];
